#ifndef MULTISTATESENSOR_H_
#define MULTISTATESENSOR_H_

#include <QQueue>
#include <QMutex>
#include <QMutexLocker>
#include <QWaitCondition>

#include "Sensor.h"

template< typename T >
class MultiStateSensor : public Sensor
{

public:

    MultiStateSensor(QString type, QString name) : Sensor(type, name) {}
	~MultiStateSensor() {}

	T getOldestData() {
		QMutexLocker lock(&mxhistory);
		return history.dequeue();
	}

	T getNewestData() {
		QMutexLocker lock(&mxhistory);
		return history.last();
	}

	void clearData() const {
		QMutexLocker lock(&mxhistory);
		history.clear();
	}

	bool hasData() const {
		QMutexLocker lock(&mxhistory);
		return !history.isEmpty();
	}

	// returns false if timed out, true otherwise
	bool waitForData(unsigned long timeout = ULONG_MAX) {
		QMutexLocker lock(&mxhistory);
		return historycondition.wait(&mxhistory, timeout);
	}


protected:
	QQueue< T > history;
	QMutex mxhistory;
	QWaitCondition historycondition;


	virtual void enqueueData(const T &data) {
		QMutexLocker lock(&mxhistory);
		history.enqueue(data);
		historycondition.wakeOne();
	}


};

#endif
