
#ifndef DEVICEMANAGER_H
#define DEVICEMANAGER_H

#include <QTcpSocket>
#include <QThread>
#include <QMap>
#include <QDataStream>
#include <QQueue>
#include <QString>
#include <QMutex>
#include <QMutexLocker>
#include <QUuid>
#include "Exception.h"
#include "Device.h"

class Robot : public QThread {

	private:
	
		QTcpSocket server;

		static const int TIMEOUT;
		static const int BUFSIZE;
		
		QString host;
		quint16 port; // q uint 16 is a 16 bytes unsigned int type defined by Qt
		
		QMap< QString, QMap< QString, Device* > > devices;
		
		QQueue< QString > output;
		QMutex outputMutex;
		
		bool keepRunning;

        QUuid uniqueID;
		QString botModel;
		QString botName;
		QString botPos;
		QString botRot;

	public:
	
		Robot(const QString &host, const quint16 port);
		

        QUuid getUniqueID();
        QString getName();
        QString getUniqueName();

		void setModel(QString botModel);
		void setName(QString botName);
		void setPos(QString botPos);
		void setRot(QString botRot);
		
		void connect(); // throws
		
		void disconnect();
		
		void stop();
		
		void addDevice(Device *device);
		
		void putCommand(QString cmd);
		
		void run();
		
	private:
	
		void spawnRobot();
		
		bool distribute(QMap<QString, QString> *map);
		
};

#endif
