#include "INUSensor.h"
#include "Agent.h"

#include <iostream>
#include <QDebug>
#include <QStringList>


const QString INUSensor::INU = "INU";


INUSensor::INUSensor(QString name)
: SingleStateSensor<INUData>(INUSensor::INU, name)
{
    QMutexLocker lock(&this->mxdata);
    this->data.droll  = this->offset.droll = 0.0;
    this->data.dpitch = this->offset.dpitch = 0.0;
    this->data.dyaw   = this->offset.dyaw = 0.0;
}

INUSensor::~INUSensor() 
{
	needsConfiguration = true;
}


void INUSensor::accept(QMap<QString, QString>* msg) {
    bool changed = false;
    if (this->isSensorMessage(msg)) {
        QMutexLocker lock(&this->mxdata);
        QStringList list = (*msg)["Orientation"].split(",");
        this->data.droll  = list[0].toDouble() - this->offset.droll;
        this->data.dpitch = list[1].toDouble() - this->offset.dpitch;
        this->data.dyaw   = list[2].toDouble() - this->offset.dyaw;
        changed = true;

		//qDebug() << "INU  " << (*msg)["Orientation"];
		//qDebug() << "DATA " << data.droll << "," << data.dpitch << "," << data.dyaw;

	} else if (this->isResponseMessage(msg)) {
        if (this->isStatusOK(msg)) {
            QMutexLocker lock(&this->mxdata);
            this->data.droll  = this->offset.droll = 0.0;
            this->data.dpitch = this->offset.dpitch = 0.0;
            this->data.dyaw   = this->offset.dyaw = 0.0;
            changed = true;
        }
    }


    if (changed)  {
        this->agent->notifyUpdate(this);
    }
}

void INUSensor::reset() {
    //this->sendSetCommand("Reset");
    QMutexLocker lock(&this->mxdata);
    this->offset = this->data;
    this->data.droll  = 0.0;
    this->data.dpitch = 0.0;
    this->data.dyaw   = 0.0;

	//qDebug() << "RESET INU";

}
