#include "NavAgent.h"

#include <QtDebug>
#include <QStringList>


NavAgent::NavAgent() : Agent()
{
}

NavAgent::~NavAgent() {
}

void NavAgent::configure(QString model, QString name, QString position, QString rotation) {
    Agent::configure(model, name, position, rotation);

    this->position   = new PositionActuator(this);
    this->keyboard   = new KeyboardSensor();
    this->navigator  = new Navigator(this->position);

    this->sonar      = new SonarSensor();
    this->rfid       = new RFIDSensor("RFID");
    this->victimrfid = new VictimRFIDSensor("VictRFID");

    this->mount(keyboard);
    this->mount(sonar);
    this->mount(rfid);
    this->mount(victimrfid);

    this->keyboard->start();

}

void NavAgent::notifyUpdate(Sensor* sensor) {
    Agent::notifyUpdate(sensor);

    if (sensor->getType() == SonarSensor::SONAR) {
        this->observed(((SonarSensor*) sensor)->getData());
    } else if (sensor->getType() == RFIDSensor::RFID) {
        this->observed(((RFIDSensor*) sensor)->getData());
    } else if (sensor->getType() == VictimRFIDSensor::VICTIMRFID) {
        this->observed(((VictimRFIDSensor*) sensor)->getData());
    }
   
}

void NavAgent::observed(SonarData range) {
    this->navigator->process(range);
}

void NavAgent::observed(QMap<QString, RFIDData> rfid) {
	QList<RFIDData> rfids = rfid.values();
	for(int i=0; i<rfids.size(); i++)
		this->navigator->process(rfids[i]);
}

void NavAgent::observed(QMap<QString, VictimRFIDData> rfid) {
	QList<VictimRFIDData> rfids = rfid.values();
	for(int i=0; i<rfids.size(); i++)
		this->navigator->process(rfids[i]);
}

