#include "Navigator.h"

#include <QtDebug>
#include <QStringList>
#include <math.h>

using namespace std;

/* SONAR CONFIGURATION:

FRONT RING:
F1 X=  7.6125  Y=-6.825      Angle=-16384  ( -90 degrees)
F2 X=  9.7125  Y=-6.0375     Angle=-9102   ( -50 degrees)
F3 X= 11.55    Y=-4.2000     Angle=-5461   ( -30 degrees)
F4 X= 12.60    Y=-1.3125     Angle=-1820   ( -10 degrees)
F5 X= 12.60    Y= 1.3125     Angle= 1820   (  10 degrees)
F6 X= 11.55    Y= 4.2000     Angle= 5461   (  30 degrees)
F7 X=  9.7125  Y= 6.0375     Angle= 9120   (  50 degrees)
F8 X=  7.6125  Y= 6.825      Angle= 16384  (  90 degrees)

BACK RING:
R1 X= -7.6125  Y= 6.825      Angle= 16384  (  90 degrees)
R2 X= -9.7125  Y= 6.0375     Angle= 23666  ( 130 degrees)
R3 X=-11.55    Y= 4.2000     Angle= 27307  ( 150 degrees)
R4 X=-12.60    Y= 1.3125     Angle= 30948  ( 170 degrees)
R5 X=-12.60    Y=-1.3125     Angle=-30948  (-170 degrees)
R6 X=-11.55    Y=-4.2000     Angle=-27307  (-150 degrees)
R7 X= -9.7125  Y=-6.0375     Angle=-23666  (-130 degrees)
R8 X= -7.6125  Y=-6.825      Angle=-16384  (- 90 degrees)
*/


Navigator::Navigator(PositionActuator* actor)
{
    this->actor = actor;
    this->canGoN = this->canGoNE = this->canGoNW = true;
}

Navigator::~Navigator() {
}

void Navigator::process(SonarData range) {
    double threshold = .5;

    this->canGoN   = min(range.F4, range.F5);

    this->canGoNNW = min(range.F4, range.F3);
    this->canGoNW  = min(range.F3, range.F2);
    this->canGoW   = min(range.F1, range.R8);

    this->canGoNNE = min(range.F5, range.F6);
    this->canGoNE  = min(range.F6, range.F7);
    this->canGoE   = min(range.F8, range.R1);

    this->canGoSW  = min(range.R6, max(range.R5, range.R7));
    this->canGoSE  = min(range.R3, max(range.R4, range.R2));
    this->canGoS   = min(range.R4, range.R5);

    this->navigate();
}


void Navigator::process(VictimRFIDData rfid) {
}


void Navigator::navigate() {
    float N = .5;

    if (this->canGoN > N) {
        qDebug() << QString("Going NORTH (%1)").arg(this->canGoN);
        this->actor->drive(N, 0);
    } else if (this->canGoNNE > N) {
        qDebug() << QString("Turn EAST (%1)").arg(this->canGoN);
        this->actor->drive(0, .1F);
    } else if (this->canGoNNW > N) {
        qDebug() << QString("Turn WEST (%1)").arg(this->canGoN);
        this->actor->drive(0, -.1F);
    }
}