#include "VictimRFIDSensor.h"
#include "Agent.h"

#include <iostream>
#include <QDebug>
#include <QStringList>


const QString VictimRFIDSensor::VICTIMRFID = "VictRFID";

const QString VictimRFIDSensor::FALSEALARM = "FalsePositive";
const QString VictimRFIDSensor::NOID       = "None";
const QString VictimRFIDSensor::NOSTATUS   = "None";



VictimRFIDSensor::VictimRFIDSensor(QString name)
: SingleStateSensor< QMap<QString, VictimRFIDData> >(VictimRFIDSensor::VICTIMRFID, name)
{
	needsConfiguration = true;
}

VictimRFIDSensor::~VictimRFIDSensor()
{
}

void VictimRFIDSensor::injectObservation(VictimRFIDData obs) {
	
	{
		QMutexLocker lock(&this->mxdata);
		this->data.clear();
		
		this->data.insert(obs.ID, obs);
	}
	this->agent->notifyUpdate(this);
}

void VictimRFIDSensor::accept(QMap<QString, QString>* msg) {

	if( isConfMessage(msg) ) {
		maxRange = getDouble(msg, "MaxRange");
		fpMinRange = getDouble(msg, "FPMinRange");
		victMinRange = getDouble(msg, "VictMinRange");
	}
	else
	{
		this->data.clear();
		{
			QMutexLocker lock(&this->mxdata);

			QList<QString> IDs = msg->values("ID");
			QList<QString> Stati = msg->values("Status");
			QList<QString> Locs = msg->values("Location");

			for(int i=0; i<IDs.size() && i<Stati.size() && i<Locs.size(); i++)
			{
				VictimRFIDData tag;
				tag.ID = IDs[i];
				tag.status = Stati[i];


				/* decision schema for level:
				 * - falsealarm      -> we see a false alarm
				 * - nostatus + noid -> we see something, maybe false alarm when we get closer
				 * - nostatus + id   -> we see a victim 
				 * - status + id     -> we see a victim + status
				 * 
				 * Note: to see the status but no id is impossible.
				 */

				if (tag.ID == VictimRFIDSensor::FALSEALARM) {
					tag.level = CANSEEFALSEALARM;
				} else if (tag.status == VictimRFIDSensor::NOSTATUS) {
					if (tag.ID == VictimRFIDSensor::NOID) {
						tag.level = CANSEE;
					} else {
						tag.level = CANSEEVICTIM;
					}
				} else {
					tag.level = CANSEEVICTIMSTATUS;
				}

				QStringList list = Locs[i].split(",");
				tag.dx = list[0].toDouble();
				tag.dy = list[1].toDouble();
				tag.dz = list[2].toDouble();

				this->data.insert(tag.ID, tag);
			}
		}
		this->agent->notifyUpdate(this);
	}
}


