
#include "CubeAgent.h"


bool CubeAgent::isOverlappingCell (
	const Cube&     item,
	const Vector3f& lowerCorner,
	const Vector3f& upperCorner
) const
{
	const Vector3f& itemLower(item.getPosition());
	const Vector3f  itemUpper = item.getPosition() + item.getDimensions();

	// check the two ranges overlap in every dimension
	return Vector3f::ONE() == (itemLower < upperCorner) &&
	       Vector3f::ONE() == (itemUpper > lowerCorner);
}


dword CubeAgent::getSubcellOverlaps (
	const Cube&     item,
	const Vector3f& cellsLowerPos,
	const Vector3f& cellsMiddlePos,
	const Vector3f& cellsUpperPos
) const
{
	// efficiency could (probably) be improved by doing minimal necessary checks
	// against the dividing bounds, instead of repeatedly delegating to
	// isOverlappingCell().

	dword flags = 0;

	const Vector3f* lowMidPoints[]  = {&cellsLowerPos,  &cellsMiddlePos};
	const Vector3f* midHighPoints[] = {&cellsMiddlePos, &cellsUpperPos};

	for(dword i = 8; i-- > 0; )	{
		Vector3f lowerCorner(lowMidPoints[ i       & 1]->getX(),
		                     lowMidPoints[(i >> 1) & 1]->getY(),
		                     lowMidPoints[(i >> 2) & 1]->getZ());
		Vector3f upperCorner(midHighPoints[ i       & 1]->getX(),
		                     midHighPoints[(i >> 1) & 1]->getY(),
		                     midHighPoints[(i >> 2) & 1]->getZ());
		flags |= dword(isOverlappingCell(item, lowerCorner, upperCorner)) << i;
	}

	return flags;
}
