#ifndef GLVIEWCANVAS_H
#define GLVIEWCANVAS_H

#include <Qt>
#include <QtOpenGL>
#include <QMouseEvent>
#include <QKeyEvent>

#include <QColor>
#include <QEvent>
#include <QList.h>
#include <QPoint.h>
#include <QTimer.h>

#include "GLViewItem.h"
#include "Util3D.h"

// ***** little class to be used to z-order the list of view items *****
class ViewListNode
{
	public:
	
		int z;
		GLViewItem * item;
		
		ViewListNode(int inZ, GLViewItem* inPtr);
		~ViewListNode();
};
bool operator<(ViewListNode &left, ViewListNode &right);
bool operator==(ViewListNode &left, ViewListNode &right);

class GLViewCanvas : public QGLWidget
{
	Q_OBJECT
	
	public:
		GLViewCanvas(QWidget* =0);
		virtual ~GLViewCanvas();

		static const int glWidth() { return mvWidth; }
		static const int glHeight() { return mvHeight; }
		static const GLfloat glNear() { return mvNear; }
		static const GLfloat glFar() { return mvFar; }
		
		void addView(int, GLViewItem*);
		
	protected:
		void initializeGL();
		void resizeGL(int,int);
		void paintGL();
		
		
		void mousePressEvent(QMouseEvent *);
		void mouseReleaseEvent(QMouseEvent *);
		void mouseMoveEvent(QMouseEvent *);
		
		void keyPressEvent(QKeyEvent *);
	
	private:

		bool mvDragFlag;
		Qt::MouseButton mvDragState;
		QPoint mvStartDragPoint;
		
		// things for rotation
		QPoint mvStartRotate;
		Camera mvCam;
		
		QList<ViewListNode*> views;
		
		static const GLfloat mvRotStep;
		
		static int mvWidth;
		static int mvHeight;
		static const GLfloat mvNear;
		static const GLfloat mvFar;
		
		// a timer to redraw
		QTimer * mvTimer;
		
		void mouseDrag(Qt::MouseButton, int, int);
		void mouseClick(Qt::MouseButton, int, int);
		void mouseStartDrag(Qt::MouseButton, int, int);
		void mouseStopDrag(Qt::MouseButton, int, int);
	
	// static "utility calls"
	public:
		
		/// sets a QColor in GL
		static void QGLColor(QColor);
	
};

#endif //GLVIEWCANVAS_H
