#include "Map.h"

#include <QMapIterator>
#include <QDebug>

Map::Map(float extend)
	: octree(extend)
{
}

Map::~Map()
{
	QMapIterator<QString, Trail*> it(trails);
	
	while(it.hasNext()) {
		it.next();
		delete it.value();
	}
	trails.clear();
}

void Map::addTrail(QString name) {
	if(!trails.contains(name)) {
		trails.insert(name,new Trail(name));
	}
}

void Map::addObservation(QString trailName, TrailObservation* obs) {
	Trail* trail = NULL;
	
	if( trails.contains(trailName) ) {
		trail = trails[trailName];
	} else {
		trail = new Trail(trailName);
		trails.insert(trailName,trail);
	}
	
	trail->addObservation(obs);
}

#ifdef USE_OPENGL
void Map::drawGL() {
	QMapIterator<QString, Trail*> it(trails);
	
	glColor3f(1,1,1);
	
	glPushMatrix();
	
	while(it.hasNext()) {
		it.next();
		it.value()->drawGL();
	}
	
	glColor3f(1,1,1);
	octree.drawGL();
	
	glColor3f(1,.2,.2);
	
	glBegin(GL_LINES);
		glVertex3d(1,1,1);
		glVertex3d(2,2,4);
		
		glVertex3d(2,2,4);
		glVertex3d(2,1,0);
		
		glVertex3d(2,1,0);
		glVertex3d(1,1,1);
	glEnd();
	
	glPopMatrix();
}

void Map::setColors(QString trailName, QColor trail, QColor scans) {
	Trail* trailObj = NULL;
	if( trails.contains(trailName) ) {
		trailObj = trails[trailName];
		trailObj->setColors(trail,scans);
	}
}
#endif
