#include "TrailObservation.h"

#include <QtDebug>

TrailObservation::TrailObservation(LaserData3D laser, Pose3D pose)
{
	this->laser = laser;
	this->poseDiff = pose;
	displayList = 0;
}

TrailObservation::~TrailObservation()
{
}

void TrailObservation::setParent(Trail* p) {
	parent = p;
}

#ifdef USE_OPENGL
void TrailObservation::drawGL() {
	
	glTranslatef(poseDiff.x,poseDiff.y,poseDiff.z);
	glRotatef(poseDiff.rotX * 180 / M_PI,1.0,0.0,0.0);
	glRotatef(poseDiff.rotY * 180 / M_PI,0.0,1.0,0.0);
	glRotatef(poseDiff.rotZ * 180 / M_PI,0.0,0.0,1.0);
	
	double x = 0, y = 0, z = 0;
	double topLeftH = -laser.hFov/2;
	double topLeftV = -laser.vFov/2;
	
	GLViewCanvas::QGLColor(parent->getScanColor());
	
	glPushMatrix();
		
		if(!displayList) {
			
			displayList = glGenLists(1);
			glNewList(displayList, GL_COMPILE_AND_EXECUTE);
			
			glBegin(GL_POINTS);
			for( int i=0; i< laser.height; i++) {
				for( int j=0; j< laser.width; j++) {
					if(laser.data[i][j] < 19.9 && laser.data[i][j] > .2) { 
						x =   laser.data[i][j] * cos(topLeftV + laser.vResolution*i) * cos(topLeftH + laser.hResolution*j);
						y =   laser.data[i][j] * cos(topLeftV + laser.vResolution*i) * sin(topLeftH + laser.hResolution*j);
						z = - laser.data[i][j] * sin(topLeftV + laser.vResolution*i);
						
						glVertex3d(x, y, z);
					}
				}
			}
			glEnd();
			
			glEndList();
			
		} else {
			glCallList(displayList);
		}
		
	glPopMatrix();
}
#endif
