#ifndef XSOCKET_H_
#define XSOCKET_H_

#include <winsock2.h>

#include <QString>
#include <QByteArray>
#include <QQueue>

#include "Exception.h"


// simple synchronous socket wrapper with standardized interface
class XSocket
{
protected:
	SOCKET sockfd;
	WSADATA wsa;

	sockaddr_in remoteAddr;
	
	bool aIsConnected;
	
	char* buf;
	unsigned int alreadyReadBefore;
	
	QQueue< QString > receiveQueue;
	QQueue< QByteArray > receiveBytesQueue;

	QQueue< QByteArray > sendWhenDisconnectedQueue;
	bool queueWhenDisconnected;
	
	int findNextCRLF(char*, int, int);
	int copyToFrontOfBuffer(int, int);
	
	int sendAll(const QByteArray&);
	
	bool waitForData(unsigned long);
	void readMore();
	
public:
	static const int BUFSIZE;

	XSocket(SOCKET);

	XSocket();
	virtual ~XSocket();
	
	virtual void connect(const QString&, uint);
	
	virtual void setQueueWhenDisconnected(bool);
	
	virtual void disconnect();
	
	virtual bool isConnected();
	
	virtual int send(const QString&);
	virtual QString receive(unsigned long);

	virtual int sendBytes(const QByteArray&);
	virtual QByteArray receiveBytes(unsigned long);
	
	virtual QString lastError();
};

#endif /*XSOCKET_H_*/
