#include "Client.h"

#include "Exception.h"

#include <QtDebug>

Conversation::Conversation()
	: ConversationId( QUuid::createUuid() )
{
}

QByteArray Conversation::getReturnValue()
{
	return returnValue;
}
void Conversation::setReturnValue(QByteArray data)
{
	returnValue = data;
}

void Conversation::suspend()
{
	mutex.lock();
	waiter.wait(&mutex);
	mutex.unlock();
}

void Conversation::resume()
{
	waiter.wakeOne();
}


Client::Client(QString pHost, uint pPort)
	: Connection( new XSocket() )
{
	host = pHost;
	port = pPort;
	listener = NULL;
}

Client::~Client()
{
}

bool Client::connect()
{
	socket->connect(host,port);
	
    qDebug() << "connected!";
    
    // enter threaded event loop
    start();
    
    return true;
}

void Client::suspend(Conversation* conv)
{
	conversations.insert(conv->getId(), conv);
	qDebug() << "suspending conv: " << conv->getId();
	conv->suspend();
}

void Client::resume(Conversation* conv)
{
	conversations.remove(conv->getId());
	qDebug() << "resuming conv: " << conv->getId();
	conv->resume();
}

void Client::setListener(ClientBroadcastListener* list)
{
	listener = list;
}

void Client::received(QUuid& id, QByteArray data)
{
	qDebug() << "received: " << id;
	if( !id.isNull() && conversations.contains(id) )
	{
		qDebug() << "resuming conversation";
		
		Conversation* conv = conversations.value(id);

		conv->setReturnValue(data);
		resume(conv);
	}
	else if(listener != NULL)
	{
		qDebug() << "no conv, broadcast!";
		
		listener->receivedBroadcast(data);
	}
	else
	{
		qDebug() << "Couldn't process message: " << id;
	}
}

//Manifold* Client::getSubMap(ScanObservation* obs)
//{
//	QByteArray msg;
//	QDataStream out(msg);
//	
//	out << "getSubMap";
//	out << obs->getUniqueID();
//	
//	Conversation conv;
//	send(conv.getId(), msg);
//	suspend(&conv);
//	
//	Manifold* man = new Manifold();
//	QByteArray* data = conv.getReturnValue();
//	
//	// so the datastream goes out of scope before the byte array
//	{
//		QDataStream in(*data);
//		in >> (*man);
//	}
//	
//	delete data;
//	return man;
//}
//	
//void Client::sendPoseUpdate(const Pose3D& pose, ScanObservation* currentPatch)
//{
//	Conversation conv;
//	QByteArray msg;
//	QDataStream out(msg);
//	out << "poseUpdate";
//	out << pose;
//	out << currentPatch->getUniqueID();
//	
//	send(conv.getId(), msg);
//}

// FOR TESTING ONLY
QString Client::echo(QString str)
{
	Conversation conv;
	QByteArray msg;
	QDataStream out(&msg, QIODevice::WriteOnly);
	out << QString("echo");
	out << str;
	
	if( send(conv.getId(),msg) )
		suspend(&conv);
	else
	{
		qDebug() << "echo: couldn't send message!";
		return QString();
	}
	
	QString resp;
	QByteArray data = conv.getReturnValue();
	
	// so the datastream goes out of scope before the byte array
	{
		QDataStream in(data);
		in >> resp;
	}
	
	return resp;
}

void Client::put(QString blip)
{
	Conversation conv;
	QByteArray msg;
	QDataStream out(&msg, QIODevice::WriteOnly);
	out << QString("put");
	out << blip;
	
	if( send(conv.getId(),msg) )
		qDebug() << "sent successfully: " << blip;
}
