#include "Connection.h"

#include <QtDebug>
#include <QDataStream>

ConversationId::ConversationId(QUuid pId)
{
	id = pId;
}

const QUuid& ConversationId::getId() const
{
	return id;
}

QUuid& ConversationId::getId()
{
	return id;
}



Connection::Connection(XSocket* pSocket)
{
	socket = pSocket;
	running = false;
}

Connection::~Connection()
{
	if(running)
		stop();
		
	qDebug() << "Connection deleting socket";
	if(socket!=NULL) {
		delete socket;
		qDebug() << "Connection deleted socket";
	} else {
		qDebug() << "Connection didn't have a socket?!?";
	}
}

void Connection::setQueueWhenDisconnected(bool queue)
{
	socket->setQueueWhenDisconnected(queue);
}

bool Connection::send(QUuid& id, const QByteArray& data)
{
	QByteArray final;
	QDataStream out(&final, QIODevice::WriteOnly);
	out.setVersion(QDataStream::Qt_4_0);
	
	out << id;
	out << data;
	
	int sent = socket->sendBytes(final);
	
	qDebug() << "sent with id: " << id << " databytes: " << data.size() << " transferbytes: " << sent;
	
	return true;
}

void Connection::disconnect()
{
	stop();

	socket->disconnect();
		
	qDebug() << "connection disconnected";
}

void Connection::run()
{
	running = true;

	QByteArray env;
	try {
		while( running && socket->isConnected() )
		{
			env = socket->receiveBytes(1000);
			if( env.size() > 0 )
			{
				readFromSocket(env);
			}
		}
	} catch (Exception e) {
		qDebug() << e.report();
	}
	running = false;
	
	if( !socket->isConnected() )
	{
		qDebug() << "disconnected!";
	}
}

bool Connection::isConnected()
{
	return socket->isConnected();
}

void Connection::stop()
{
	if( !running )
		return;
	
	running = false;
	
	if( isRunning() ) // if the thread is still running
		wait();
}


void Connection::readFromSocket(QByteArray envelope)
{
    QDataStream in2(envelope);
    
	QUuid id;
	QByteArray data;
	
	in2 >> id;
	in2 >> data;
	
	qDebug() << "id received: " << id << " databytes: " << data.size();
	
	received(id, data);
}
