#include "UsarConnection.h"
#include "DataOperators.h"

#include <QtDebug>

UsarConnection::UsarConnection(XSocket* sock, UsarServer* par)
	: ServerConnection(sock, par), slam(&matcher,(par->manifold))
{
	parent = par;
	hasSubmittedPatches = false;
}

UsarConnection::~UsarConnection()
{
	qDebug() << "UsarConnection destroyed";
}

void UsarConnection::received(QUuid& id, QByteArray data)
{
	quint8 code = 0;
	QDataStream in(data);
	
	in >> code;
	
	if( code == 1 )
	{
		QString ID;
		QString status;
		
		in >> ID;
		in >> status;
		
		qDebug() << "*************  new status " << status << " for id " << ID;
		
		parent->manifold->updateVictimStatus(ID,status);
	}
	else if( code == 2 )
	{
		QUuid patchId;
		
		in >> patchId;
		
		ScanObservation* obs = new ScanObservation(patchId);
		
		in >> obs->rawpose.x;
		in >> obs->rawpose.y;
		in >> obs->rawpose.rot;
		
		in >> obs->pose.x;
		in >> obs->pose.y;
		in >> obs->pose.rot;
		
		in >> obs->maxdistance;
		in >> obs->mindistance;
		
		in >> obs->unit;
		in >> obs->length;
		
		obs->range = new float[obs->length];
		for( int i=0; i<obs->length; i++)
		{
			in >> obs->range[i];
		}
		
		int countRFIDs;
		in >> countRFIDs;
		
		RFIDData rfid;
		for( int i=0; i<countRFIDs; i++)
		{
			in >> rfid;
			obs->addRFID(rfid);
		}
		
		int countVictimRFIDs;
		in >> countVictimRFIDs;
		
		VictimRFIDData rfidVictim;
		for( int i=0; i<countVictimRFIDs; i++)
		{
			in >> rfidVictim;
			obs->addVictim(rfidVictim);
		}
		
		slam.extendWithPatch(obs);
		
		qDebug() << "added new patch with id " << patchId;
		
		bool hasConnection;
		in >> hasConnection;
		
		if(hasConnection)
		{
			QUuid otherId;
			in >> otherId;
			
			Matrix3 cov;
			
			for(int i=0; i<3; i++)
			for(int j=0; j<3; j++)
				in >> cov(i,j);
			
			slam.extendWithLink(patchId, otherId, cov);
			
			qDebug() << "and it had a connection to " << otherId;
		}
		
		if( !hasSubmittedPatches )
		{
			QUuid id;
			send(id, QByteArray());
			
			parent->manifold->insertStartNode(patchId);
			qDebug() << "added " << patchId << " as start node";
			hasSubmittedPatches = true;
		}
	}
	
	// later
	// slam.addNode( ... )
}

