#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

class QAction;
class QMenu;
class QWorkspace;
class QSignalMapper;
class MapWindow;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow();

protected:
    void closeEvent(QCloseEvent *event);

private slots:

    void open();
    void save();
    void saveAs();


    void useNoSM();
    void useWeightedSM();
    void useUnweightedSM();


    void zoomIn();
    void zoomOut();
    void zoomReset();

    void about();

    void updateMenus();
    void updateWindowMenu();

    MapWindow *createMapWindow();

private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();

    MapWindow *activeMapWindow();
    MapWindow *findMapWindow(const QString &fileName);

    QWorkspace *workspace;
    QSignalMapper *windowMapper;

    QMenu *fileMenu;
    QMenu *scanMenu;
    QMenu *viewMenu;
    QMenu *windowMenu;
    QMenu *helpMenu;
    QToolBar *fileToolBar;
    QToolBar *viewToolBar;

    QAction *separatorAct;

    QAction *openAct;
    QAction *saveAct;
    QAction *saveAsAct;
    QAction *exitAct;

    QAction *useNoSMAct;
    QAction *useWeightedSMAct;
    QAction *useUnweightedSMAct;

    QAction *zoomInAct;
    QAction *zoomOutAct;
    QAction *zoomResetAct;

    QAction *closeAct;
    QAction *closeAllAct;
    QAction *tileAct;
    QAction *cascadeAct;
    QAction *arrangeAct;
    QAction *nextAct;
    QAction *previousAct;

    QAction *aboutAct;
};

#endif
