
#ifndef MAPWINDOW_H
#define MAPWINDOW_H

#include <QtGui>

#include "QtManifoldRenderer.h"

#include "SlamStrategy.h"
#include "SimpleSlam.h"
#include "KalmanSlam.h"
#include "ManifoldSlam.h"

#include "ScanMatcher.h"
#include "WeightedScanMatcher.h"
#include "UnweightedScanMatcher.h"

#include "ScanPlayer.h"
#include "LogScanPlayer.h"
#include "LiveScanPlayer.h"



class MapWindow : public QScrollArea
{
    Q_OBJECT

private:
    static const double DEFAULTWIDTH;
    static const double DEFAULTHEIGHT;
    static const double DEFAULTZOOM;

public:
    MapWindow();
    ~MapWindow();

    bool loadImageFile(const QString &fileName);
    bool loadLogFile(const QString &fileName);

    bool save();
    bool saveAs();
    bool saveFile(const QString &fileName);

    void setScanMatcher(ScanMatcher* matcher);
    void setScanPlayer(ScanPlayer* player);

    void zoomBy(double factor);
    void zoomTo(double scale);

    QString userFriendlyCurrentFile();
    QString currentFile() { return curFile; }

protected:
    void paintEvent(QPaintEvent* event);
    void closeEvent(QCloseEvent* event);

private slots:
    void documentWasModified();

private:

    bool maybeSave();
    void setCurrentFile(const QString &fileName);
    QString strippedName(const QString &fullFileName);

    QString curFile;
    bool isUntitled;


    void adjustScrollBar(QScrollBar *scrollBar, double scale);

    ScanMatcher* matcher;
    ScanPlayer*  player;


    QLabel* mapImageLabel;
    QImage mapImage;
    double zoomScale;
};

#endif
