#ifndef WEIGHTEDSCANMATCHER_H_
#define WEIGHTEDSCANMATCHER_H_

#include "ScanMatcher.h"
#include "Manifold.h"

class WeightedScanMatcher : public ScanMatcher
{

private:

    // indices into angle arrays
    static const uint ANG = 0;
    static const uint COS = 1;
    static const uint SIN = 2;

    // maximum distance within which to detect correlations
    static const uint CORRELATION_MAXDISTANCE = 2000;

    // tweak stop and break conditions
    static const uint MAXITERATIONS     = 80; // max iteration count
    static const uint RESULT_COUNT      = 10; // max num of results to consider 
    static const uint RESULT_BREAKCOUNT =  8; // how many good results before converged

    // only results below the error threshold are considered
    static const float ERROR_THRESHOLD;


    // these values are further explained in the cpp file
    static const float IANGLE_THRESHOLD;
    static const float DEFAULT_SIGMAANG;
    static const float DEFAULT_SIGMADIST;
    static const float ANGLE_INCREMENT;
    static const float SIGMA_CORRELATED;

    

public:
	
	WeightedScanMatcher();
	virtual ~WeightedScanMatcher();
		
    MatchResult* matchScans(QList<ScanObservation*>* submap, ScanObservation* primary, ScanObservation* scan2);
    MatchResult* matchScans(ScanObservation* scan1, ScanObservation* scan2);

private:

    // since it is not possible to determine vector lengths run-time, we need this
    // pseudo-constant that tells us the currently applicable vector length
    uint NUMSCANS;
    uint LENGTH;
    uint LENGTH1;
    uint LENGTH2;


    // all scan distances out of this range are ignored
    uint RANGESCAN_MINDISTANCE;
    uint RANGESCAN_MAXDISTANCE;


    MatchResult* matchPoints(Vector2* points1, Vector2* points2, Pose2D odometry);



    Vector2* toPoints(QList<ScanObservation*>* scans, ScanObservation* primary, float** angles);
    Vector2* toPoints(ScanObservation* scan, float** angles);
    Vector2* toPose(Vector2* points, Pose2D pose);
    float    toAngle(Vector2 point);
    float    toDistance(Vector2 point);

    Pose2D getRelativePose(Pose2D pose1, Pose2D pose2);

    uint**  correlatePoints(Vector2* points1, Vector2* points2, bool* filter1, bool* filter2, uint& corrcount);

    float** computeAngles(float fromAngle, float toAngle, float resolution);
    float** computeIncidenceAngles(Vector2* points, uint numScans);

    Matrix2 computeErrorCovariance(Vector2 point, float* iangle);
    Matrix2 computeNoiseErrorCovariance(Vector2 point);
    Matrix2 computeCorrespondenceErrorCovariance(Vector2 point, float* iangle);


};

#endif
