#include "time.h"

#include "ManifoldSlam.h"


//const float ManifoldSlam::EXT_TRANSLATIONCOVARIANCE = 5.0F;
//const float ManifoldSlam::EXT_ROTATIONCOVARIANCE    = M_PI / 16.0F;
const float ManifoldSlam::EXT_PATCHPOSEDISTANCE     = 1.00F;
const float ManifoldSlam::EXT_PATCHPOSEROTATION     = 30.0F * M_PI / 180;
const float ManifoldSlam::EXT_TRANSLATIONCOVARIANCE = 10.0F;
const float ManifoldSlam::EXT_ROTATIONCOVARIANCE    = M_PI / 16.0F;
const uint  ManifoldSlam::EXT_NUMCORRESPONDENCIES   = 140;

const float ManifoldSlam::MAX_TRANSLATIONCOVARIANCE = 100.0F;
const float ManifoldSlam::MAX_ROTATIONCOVARIANCE    = M_PI / 2.0F;
const uint  ManifoldSlam::MAX_NUMCORRESPONDENCIES   = 30;

ManifoldSlam::ManifoldSlam(ScanMatcher* matcher, Manifold* manifold)
: SlamStrategy(matcher)
{
    this->manifold = manifold;
    this->currentSubmap = 0;
    this->currentPatch = 0;
    this->number = 0;
    this->iMax = 0;
}

ManifoldSlam::~ManifoldSlam()
{
    if (this->currentSubmap)
        this->currentSubmap->clear();
}

ScanObservation* ManifoldSlam::getCurrentPatch() {
    return this->currentPatch;
}
Pose2D ManifoldSlam::getCurrentPose() {
    return this->currentPose;
}

void ManifoldSlam::addListener(ManifoldSlamListener* list) {
	listeners.append(list);
}

bool ManifoldSlam::processScan(ScanObservation* scan) 
{
	// return value
	bool changedPatch = false;

    this->number++;

    if (!this->currentPatch) {
        // first round, initialize current pose 
        this->currentPose.x   = scan->pose.x;
        this->currentPose.y   = scan->pose.y;
        this->currentPose.rot = scan->pose.rot;

        // initialize the manifold at this patch
        this->extendManifold(scan);
        
        for(int i=0; i<listeners.size(); i++)
			listeners[i]->newPatchAdded(scan);
        
        this->moveToPatch(scan);

		qDebug() << "-- MANIFOLDSLAM INIT (" << this->manifold->nodeCount() << " nodes / " << this->manifold->linkCount() << " links)";

        changedPatch = true;

    } else {
        // match this scan against the current patch
        time_t       start  = clock();
        MatchResult* result = this->matcher->matchScans(currentSubmap, currentPatch, scan);
        //MatchResult* result = this->matcher->matchScans(currentPatch, scan);
        time_t       stop   = clock();

        double elapsed = (double) (stop - start) / CLOCKS_PER_SEC;

        this->nSum += result->numIterations;
        if (result->numIterations > this->nMax || this->iMax == 0) {
            this->iMax = number;
            this->nMax = result->numIterations;
            this->sMax = elapsed;
        }

        qDebug() << "-- " << result->numIterations << " iterations / "  << result->numPatchesInSubmap << " patches / "  << result->numCorrespondencies << " correspondencies / took " << elapsed << " secs";
        qDebug() << "-- sensor dpose     = " << result->rawdpose.x << result->rawdpose.y << result->rawdpose.rot;
        qDebug() << "-- weighted dpose   = " << result->smdpose.x << result->smdpose.y << result->smdpose.rot << "(" << result->numIterations << " iterations)";
        qDebug() << "-- covariance trace = " << result->mcov(0, 0) << "/" << result->mcov(1, 1) << "/" << result->mcov(2, 2); 

        // the returned dpose is wrt the currentPatch
        // so here it is converted to map (global) coords
        Matrix3 rotmx;
        Vector2 vdpose(result->smdpose.x, result->smdpose.y);
        rotmx.setRotation(this->currentPatch->pose.rot);
        vdpose = rotmx.multiply(vdpose);

        // update current (global) pose
        this->currentPose.x   = this->currentPatch->pose.x   + vdpose.x;
        this->currentPose.y   = this->currentPatch->pose.y   + vdpose.y;
        this->currentPose.rot = this->currentPatch->pose.rot + result->smdpose.rot;



        // check if we should extend the manifold
        bool extend = false;
		extend = extend || (pow(result->smdpose.x, 2) + pow(result->smdpose.y, 2)) > pow(ManifoldSlam::EXT_PATCHPOSEDISTANCE, 2);
		extend = extend || (abs(result->smdpose.rot) > ManifoldSlam::EXT_PATCHPOSEROTATION);
        extend = extend || (result->mcov(0, 0) > ManifoldSlam::EXT_TRANSLATIONCOVARIANCE);
        extend = extend || (result->mcov(1, 1) > ManifoldSlam::EXT_TRANSLATIONCOVARIANCE);
        extend = extend || (result->mcov(2, 2) > ManifoldSlam::EXT_ROTATIONCOVARIANCE);
		extend = extend || (result->numCorrespondencies < EXT_NUMCORRESPONDENCIES);

		//extend = true;

        if (extend || scan->hasRFIDs()) {

            scan->pose = this->currentPose;

			bool startisland = false;
			startisland = startisland || (result->mcov(0, 0) > ManifoldSlam::MAX_TRANSLATIONCOVARIANCE);
			startisland = startisland || (result->mcov(1, 1) > ManifoldSlam::MAX_TRANSLATIONCOVARIANCE);
			startisland = startisland || (result->mcov(2, 2) > ManifoldSlam::MAX_ROTATIONCOVARIANCE);
			startisland = startisland || (result->numCorrespondencies < MAX_NUMCORRESPONDENCIES);
			//startisland = startisland || ( ( result->smdpose.x * result->smdpose.x + result->smdpose.y * result->smdpose.y) > 1000*1000 );

			if (startisland) {
				this->extendManifold(scan);
				
				for(int i=0; i<listeners.size(); i++)
					listeners[i]->newPatchAdded(scan);
				
				qDebug() << "-- MANIFOLDSLAM ISLAND (" << this->manifold->nodeCount() << " nodes / " << this->manifold->linkCount() << " links)";
			} else {
	            this->extendManifold(scan, this->currentPatch, result->mcov);
	            
	            for(int i=0; i<listeners.size(); i++)
					listeners[i]->newPatchAdded(scan, this->currentPatch->getUniqueID(), result->mcov);
	            
				qDebug() << "-- MANIFOLDSLAM EXTEND (" << this->number << " scans / " << this->manifold->nodeCount() << " nodes / " << this->manifold->linkCount() << " links)";
			}

            this->moveToPatch(scan);

			changedPatch = true;
            
		} else {
			// localize the robot
			//if (this->localize()) {
			//	qDebug() << "-- MANIFOLDSLAM LOCALIZE (" << this->manifold->nodeCount() << " nodes / " << this->manifold->linkCount() << " links)";
			//}			
		}

        delete result;
        result = 0;
    }

	// return to caller if this scan observation resulted in 
	// the robot having moved to another patch on the manifold
    return changedPatch;
}

// CALLED ONLY ON THE SERVER
void ManifoldSlam::extendWithPatch(ScanObservation* newPatch)
{
	GraphWriteLocker(this->manifold);
	this->manifold->insertPatch(newPatch);
}

// CALLED ONLY ON THE SERVER
void ManifoldSlam::extendWithLink(QUuid firstId, QUuid secondId, Matrix3 cov)
{
	GraphWriteLocker(this->manifold);
	this->manifold->insertRelation(firstId, secondId, cov);
}

void ManifoldSlam::extendManifold(ScanObservation* newPatch) 
{
    this->manifold->insertPatch(newPatch);
}

void ManifoldSlam::extendManifold(ScanObservation* newPatch, ScanObservation* atPatch, const Matrix3& mcov) 
{
    this->manifold->insertPatch(newPatch);
    if (atPatch != NULL) {
        this->manifold->insertRelation(atPatch, newPatch, mcov);
    }
}
void ManifoldSlam::moveToPatch(ScanObservation* patch) {
    this->currentPatch = patch;
    if (this->currentSubmap)
        this->currentSubmap->clear();
    this->currentSubmap = this->getSubmap(this->currentPatch);
}

QList<ScanObservation*>* ManifoldSlam::getSubmap(ScanObservation* patch) {
    QList<ScanObservation*>* patches = new QList<ScanObservation*>();

    Manifold* submap = this->manifold->getSubMap(patch);
    QMapIterator<QUuid, GraphNode*> nodeIterator = submap->nodeIterator();
    while (nodeIterator.hasNext()) {
        nodeIterator.next();
        patches->append(((ManifoldNode*) nodeIterator.value())->patch);
    }
    delete submap;

    return patches;
}


bool ManifoldSlam::localize() {

    Matrix3          rotmx;
    Vector2          vpose;
	double           dist;
	double           minDist  = -1;
	ScanObservation* minPatch = 0;

	ScanObservation* patch;
    for (uint s = 0; s < (uint) this->currentSubmap->count(); s++) {
        patch = this->currentSubmap->at(s);

		// get rotation matrix to map global pose to this patch
		rotmx.loadIdentity();
		rotmx.invFromPose2D(patch->pose);

		// transform current pose
		vpose = Vector2(this->currentPose.x, this->currentPose.y);
		vpose = rotmx.multiply(vpose);

		// check if we could be on this patch
		if (vpose.y >= -5.0) {
			// maintain the closest one
			dist = sqrt(pow((vpose.x - patch->pose.x), 2) + pow((vpose.y - patch->pose.y), 2));
			if (dist < minDist || minDist < 0) {
				minDist  = dist;
				minPatch = patch;
			}
		}
	}
	if (minPatch != NULL && minPatch->getUniqueID() != this->currentPatch->getUniqueID()) {
		this->moveToPatch(minPatch);
		return true;
	}
	return false;
}




void ManifoldSlam::closeLoop(ScanObservation* source, ScanObservation* target) {

    Matrix3 mcov;
    ManifoldNode* sourceNode = (ManifoldNode*) this->manifold->getNode(source->getUniqueID());
    ManifoldNode* targetNode = (ManifoldNode*) this->manifold->getNode(target->getUniqueID());
    this->manifold->insertRelation(source, target, mcov);
    this->refitPatches(sourceNode, targetNode);
    
    QList<QUuid> done;
    done.append(sourceNode->getUniqueID());
    done.append(targetNode->getUniqueID());

    this->refitPatches(sourceNode, done);
}

bool ManifoldSlam::mergeIslands(QUuid source, QUuid target) {

    ManifoldNode* targetNode = (ManifoldNode*) this->manifold->getNode(target);
    ManifoldNode* sourceNode = (ManifoldNode*) this->manifold->getNode(source);
	Manifold*     island1    = this->manifold->getConnectedSubSet(targetNode);
	Manifold*     island2    = this->manifold->getConnectedSubSet(sourceNode);

	QList< RFIDData > rfids1 = island1->getRFIDs();
	QList< RFIDData > rfids2 = island2->getRFIDs();
	QList< QString >  pairs;
	QList< RFIDData > pair1;
	QList< RFIDData > pair2;

	RFIDData rfid1;
	RFIDData rfid2;
	for (int i = 0; i < rfids1.count(); i++) {
		rfid1 = rfids1[i];
		if (island2->hasRFID(rfid1.ID)) {
			pairs.append(rfid1.ID);
			pair1.append(rfid1);
			for (int j = 0; j < rfids2.count(); j++) {
				rfid2 = rfids2[j];
				if (rfid2.ID == rfid1.ID) {
					pair2.append(rfid2);
					break;
				}
			}
		}
	}

	// acquire correllated points
	int count = pairs.count();
	if (count < 2) {
		return false;
	}

	Vector2 center1, center2;
	center1.x = center1.y = center2.x = center2.y = 0;
	for (int i = 0; i < pairs.count(); i++) {
		rfid1 = pair1[i];
		rfid2 = pair2[i];
		center1.x += (rfid1.dx / pairs.count());
		center1.y += (rfid1.dy / pairs.count());
		center2.x += (rfid2.dx / pairs.count());
		center2.y += (rfid2.dy / pairs.count());
	}

	double rot = 0;

	Vector2 p1, p2;
	float a1, a2, adiff;
	for (int i = 0; i < pairs.count(); i++) {
		rfid1 = pair1[i];
		rfid2 = pair2[i];
		p1 = Vector2(rfid1.dx - center1.x, rfid1.dy - center1.y);
		p2 = Vector2(rfid2.dx - center2.x, rfid2.dy - center2.y);
		a1 = atan2(p1.y, p1.x);
		a2 = atan2(p2.y, p2.x);
		adiff = a1 - a2;
		if (adiff > M_PI) {
			adiff -= 2 * M_PI;
		} else if (adiff < - M_PI) {
			adiff += 2 * M_PI;
		}
		rot += (adiff / pairs.count());
	}

	Pose2D dpose;
	dpose.x = 0;
	dpose.y = 0;
	dpose.rot = rot;

	sourceNode->moveAllConnectedBy(center2,dpose);

	pairs.clear();
	pair1.clear();
	pair2.clear();

	rfids1 = island1->getRFIDs();
	rfids2 = island2->getRFIDs();
	for (int i = 0; i < rfids1.count(); i++) {
		rfid1 = rfids1[i];
		if (island2->hasRFID(rfid1.ID)) {
			pairs.append(rfid1.ID);
			pair1.append(rfid1);
			for (int j = 0; j < rfids2.count(); j++) {
				rfid2 = rfids2[j];
				if (rfid2.ID == rfid1.ID) {
					pair2.append(rfid2);
					break;
				}
			}
		}
	}

	center1.x = center1.y = center2.x = center2.y = 0;
	for (int i = 0; i < pairs.count(); i++) {
		rfid1 = pair1[i];
		rfid2 = pair2[i];
		center1.x += (rfid1.dx / pairs.count());
		center1.y += (rfid1.dy / pairs.count());
		center2.x += (rfid2.dx / pairs.count());
		center2.y += (rfid2.dy / pairs.count());
	}

	dpose.x = center1.x - center2.x;
	dpose.y = center1.y - center2.y;
	dpose.rot = 0;

	dpose.x *= 1000;
	dpose.y *= 1000;

	sourceNode->moveAllConnectedBy(center2,dpose);


	// NOW COMPUTE QUALITY SCORE

	float error=0;
	int numCorr=0;
	float xdiff=0, ydiff=0;
	
	rfids1 = island1->getRFIDs();
	rfids2 = island2->getRFIDs();
	for (int i = 0; i < rfids1.count(); i++) {
		rfid1 = rfids1[i];
		if (island2->hasRFID(rfid1.ID)) {
			for (int j = 0; j < rfids2.count(); j++) {
				rfid2 = rfids2[j];
				if (rfid2.ID == rfid1.ID) {
					
					xdiff = rfid1.dx-rfid2.dx;
					ydiff = rfid1.dy-rfid2.dy;
					
					error += xdiff*xdiff + ydiff*ydiff;
					numCorr++;
					
					break;
				}
			}
		}
	}
	
	error /= numCorr;
	
	qDebug() << "Matching " << target << " to " << source << " has mse: " << error << " with " << numCorr << " corrlated tags";

	if( error > .5 ) // more than 50 cm off
	{
		return false; // pretend no matching;
	}

	/*
	Vector2* points1 = new Vector2[pairs.count()];
	Vector2* points2 = new Vector2[pairs.count()];

	ManifoldNode*  root1 = NULL;
	ManifoldNode*  root2 = NULL;

	Matrix3 toRoot1;
	Matrix3 toRoot2;
	Vector2 vpoint;

	for (int i = 0; i < pairs.count(); i++) {

		rfid1 = pair1[i];
		rfid2 = pair2[i];

		// add point for island1
		vpoint = Vector2(rfid1.dx, rfid1.dy);
		vpoint = toRoot1.multiply(vpoint);
		points1[i] = vpoint;

		// add point for island2
		vpoint = Vector2(rfid2.dx, rfid2.dy);
		vpoint = toRoot2.multiply(vpoint);
		points2[i] = vpoint;
	}
	//*/
	return true;
}

void ManifoldSlam::refitPatches(ManifoldNode* node, QList<QUuid>& done) {

    QMapIterator<QUuid, GraphNode*> iterator = node->nodeIterator();
    QList<ManifoldNode*> nextNodes;
    ManifoldNode* nextNode;

    while (iterator.hasNext()) {
        iterator.next();
        nextNode = (ManifoldNode*) iterator.value();
        if (!done.contains(nextNode->getUniqueID())) {
            this->refitPatches(nextNode, node);
            done.append(nextNode->getUniqueID());
            nextNodes.append(nextNode);
        }
    }

    for (int i = 0; i < nextNodes.count(); i++) {
        nextNode = (ManifoldNode*) nextNodes.at(i);
        this->refitPatches(nextNode, done);
    }
}

void ManifoldSlam::refitPatches(ManifoldNode* sourceNode, ManifoldNode* targetNode) {

    ScanObservation* sourcePatch = sourceNode->patch;
    ScanObservation* targetPatch = targetNode->patch;
    QList<ScanObservation*>* targetPatches = this->getSubmap(targetPatch);

    //MatchResult*  result = this->matcher->matchScans(targetPatches, targetPatch, sourcePatch);
    MatchResult*  result = this->matcher->matchScans(targetPatch, sourcePatch);
    ManifoldLink* link   = (ManifoldLink*) sourceNode->getLink(targetNode);

    link->mcov = result->mcov;

    Matrix3 rotmx;
    Vector2 vdpose(result->smdpose.x, result->smdpose.y);
    rotmx.setRotation(targetPatch->pose.rot);
    vdpose = rotmx.multiply(vdpose);

    sourcePatch->pose.x   = targetPatch->pose.x   + vdpose.x;
    sourcePatch->pose.y   = targetPatch->pose.y   + vdpose.y;
    sourcePatch->pose.rot = targetPatch->pose.rot + result->smdpose.rot;

}




void ManifoldSlam::processVictim(VictimRFIDData victim) {
    if (victim.level <= CANSEEFALSEALARM)
        return;

    this->manifold->updateVictimStatus(victim.ID, victim.status);
    
	for(int i=0; i<listeners.size(); i++)
		listeners[i]->newVictimStatus(victim.ID, victim.status);  
		
	qDebug() << "******** STATUS: " << victim.ID << victim.status;
}




/*
void ManifoldSlam::render(ManifoldRenderer* renderer) {

	GraphReadLocker lock(manifold);

    QMapIterator<QUuid, GraphNode*> nodeIterator = this->manifold->nodeIterator();
    ManifoldNode* node1;
	uint i = 1;
    while (nodeIterator.hasNext()) {
        nodeIterator.next();
        node1 = (ManifoldNode*) nodeIterator.value();
		renderer->renderScan(node1->patch->pose, node1->patch, i);
		i++;
    }
    
    QMapIterator<QUuid, GraphLink*> linkIterator = this->manifold->linkIterator();
    ManifoldLink* link;
    ManifoldNode* node2;
    while( linkIterator.hasNext()) {
    		linkIterator.next();
    		link  = (ManifoldLink*) linkIterator.value();
    		node1 = (ManifoldNode*) link->node1();
    		node2 = (ManifoldNode*) link->node2();
		renderer->renderLink(node1->patch->pose, node2->patch->pose);
    }
    
    QList<RFIDData> rfids = this->manifold->getRFIDs();
    for(int i=0; i<rfids.size(); i++)
    {
    		renderer->renderRFID(rfids[i]);
    }
    
    QList<VictimRFIDData> vicrfids = this->manifold->getVictimRFIDs();
    for(int i=0; i<vicrfids.size(); i++)
    {
    		renderer->renderVictimRFID(vicrfids[i]);
    }
}
*/
