#include <QtGui>
#include <math.h>

#include "MapView.h"
#include "Util3d.h"

MapView::MapView(QWidget *parent)
: QWidget(parent)
{
}

QSize MapView::minimumSizeHint() const { 
    return QSize(200, 200); 
}

QSize MapView::sizeHint() const { 
    return QSize(500, 350); 
}

void MapView::addScan(ScanObservation &scan) {
    this->scans.append(scan);
    this->update();
}

void MapView::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.fillRect(event->rect(), QBrush(Qt::white));

    plotMap(painter);
    
}

void MapView::plotMap(QPainter &painter)
{
    QPen pen(Qt::black);
    pen.setWidth(2);
    painter.setPen(pen);

    for (int i = 0; i < this->scans.count(); i++) {
        plotScan(painter, this->scans[i]);
    }
}

void MapView::plotScan(QPainter &painter, ScanObservation &scan) {

    Pose2D pose     = scan.getPose();
    LaserData range = scan.getRange();

    double alpha = pose.rot;
    double c = cos(alpha);
    double s = sin(alpha);

    double l;
    double a;
    qreal x;
    qreal y;

    this->plotPose(painter, pose);

    /*
    for (int i = 0; i < range.length; i++) {
        l = range.data[i];
        a = M_PI / 180 * i - M_PI_2;
        x = pose.x + l * cos(a);
        y = pose.y + l * sin(a);
        painter.drawPoint(QPointF(x, y));
    }
    //*/
}


void MapView::plotPose(QPainter &painter, Pose2D& pose) {

    qreal x = pose.x;
    qreal y = pose.y;

    QPen pen(Qt::red);
    pen.setWidth(2);
    painter.setPen(pen);
    painter.drawPoint(x, y);

}



