#include "QtManifoldRenderer.h"

const float QtManifoldRenderer::MAPUNIT = 100.0F;
const float QtManifoldRenderer::OFFSETX = 650.0F;
const float QtManifoldRenderer::OFFSETY = 800.0F;

QtManifoldRenderer::QtManifoldRenderer(QImage* image, Bounds bounds)
	: painter(image)
{
    this->image = image;
    
    painter.end();
    
    QPainter filler(image);
    filler.fillRect(0,0,image->width(),image->height(), QBrush(QColor(125,125,125)));
    filler.end();
    
    painter.begin(image);
    
    //this->mapToImage.translate( 0, image->height() );
    this->mapToImage.scale( image->width()/bounds.width, image->height()/bounds.height );
    this->mapToImage.translate( -bounds.left, -bounds.top );
    
    freeAreaPen.setColor(QColor(255,255,255,20));
    freeAreaPen.setWidth(5); // 4mm
    occupiedAreaPen.setColor(QColor(0,0,0,40));
    occupiedAreaPen.setWidth(20); // 10 mm
    
    rfidPen.setColor(QColor(0,0,255));
    rfidPen.setWidth(10);
    
    victimPen.setColor(QColor(255,0,0));
    victimPen.setWidth(10);

	posePen.setColor(QColor(255,50,50));
    posePen.setWidth(40); // 2 units in image or map coords??
    linkPen.setColor(QColor(50,255,50,200));
    
    this->painter.setMatrix(this->mapToImage);
    
    QFont f = painter.font();
    f.setPixelSize(100);
    painter.setFont(f);

	QPen axisPen;
	axisPen.setColor(Qt::cyan);
	painter.setPen(axisPen);
	painter.drawLine(0,bounds.height,0,-bounds.height);
	painter.drawLine(bounds.width,0,-bounds.width,0);

	for(int x = floor(bounds.left) - ((int)floor(bounds.left))%1000; x+1000<bounds.left+bounds.width; x+=1000)
	{
		painter.drawLine(x,-400, x, 400);
	}

	for(int y = floor(bounds.top) -  ((int)floor(bounds.top))%1000; y+1000<bounds.top+bounds.height; y+=1000)
	{
		painter.drawLine(-400,y, 400, y);
	}
    
    // commenting this out saves 50% processing time
    //this->painter.setRenderHint(QPainter::Antialiasing);
}

QtManifoldRenderer::~QtManifoldRenderer()
{
	painter.end();
}

void QtManifoldRenderer::renderRFID(RFIDData rfid)
{
	painter.setPen(rfidPen);
	
	rfid.dx*=1000;
	rfid.dy*=1000;
	
	painter.drawLine( rfid.dx-100, rfid.dy-100, rfid.dx+100, rfid.dy+100);
	painter.drawLine( rfid.dx-100, rfid.dy+100, rfid.dx+100, rfid.dy-100);
	
	painter.drawText(rfid.dx+100, rfid.dy+100, rfid.ID);
}

void QtManifoldRenderer::renderVictimRFID(VictimRFIDData rfid)
{
	painter.setPen(victimPen);
	
	rfid.dx*=1000;
	rfid.dy*=1000;
	
	painter.drawLine( rfid.dx-100, rfid.dy-100, rfid.dx+100, rfid.dy+100);
	painter.drawLine( rfid.dx-100, rfid.dy+100, rfid.dx+100, rfid.dy-100);
	
	painter.drawText( rfid.dx+100, rfid.dy+100, rfid.ID + " ("+rfid.status+")");
}

void QtManifoldRenderer::renderScansRFIDs(Pose2D scanPose, ScanObservation* scan, uint number) {
	/*
	 Matrix3 rotmx;
	rotmx.rotate(scanPose.rot);
    rotmx.translate(scanPose.x, scanPose.y);
    
    QMapIterator< QString, RFIDData > rfidsOnPatch = scan->rfidIterator();
    while(rfidsOnPatch.hasNext() )
    {
    		rfidsOnPatch.next();
    		
    		Vector2 rfid( rfidsOnPatch.value().dx*1000, rfidsOnPatch.value().dy*1000 );
    		rfid = rotmx.multiply( rfid );
    		
    		//qDebug() << "rfid on " << rfid.x << rfid.y << " on patch at " << scanPose.x << scanPose.y << scanPose.rot;
    		
    		painter.setPen(rfidPen);
    		painter.drawLine( rfid.x - 50, rfid.y, rfid.x + 50, rfid.y );
    		painter.drawLine( rfid.x, rfid.y - 50, rfid.x, rfid.y + 50 );
    		
    		painter.drawText( rfid.x+100, rfid.y+100, rfidsOnPatch.value().ID );
    		
    		painter.drawLine( scanPose.x, scanPose.y, rfid.x, rfid.y);
    }
    
    QMapIterator< QString, VictimRFIDData > victimsOnPatch = scan->victimIterator();
    while(victimsOnPatch.hasNext() )
    {
    		victimsOnPatch.next();
    		
    		Vector2 rfid( victimsOnPatch.value().dx*1000, victimsOnPatch.value().dy*1000 );
    		rfid = rotmx.multiply( rfid );
    		
    		painter.setPen(victimPen);
    		painter.drawLine( rfid.x - 50, rfid.y, rfid.x + 50, rfid.y );
    		painter.drawLine( rfid.x, rfid.y - 50, rfid.x, rfid.y + 50 );
    		painter.drawText( rfid.x+100, rfid.y+100, victimsOnPatch.value().ID + " ("+victimsOnPatch.value().status+")");
		
		painter.drawLine( scanPose.x, scanPose.y, rfid.x, rfid.y);    
    
    }
    */
}

void QtManifoldRenderer::renderLink(Pose2D firstPose, Pose2D secondPose) {
	painter.setPen(posePen);
	painter.drawPoint(firstPose.x, firstPose.y);
	painter.drawPoint(secondPose.x, secondPose.y);
	
	painter.setPen(linkPen);
	painter.drawLine(firstPose.x, firstPose.y, secondPose.x, secondPose.y);
}

void QtManifoldRenderer::renderScan(Pose2D scanPose, ScanObservation* scan, uint number) {

    float a = 0, step = 0;
	
	Vector2 vLocalLeft;
	Vector2 vLocalMid;
	Vector2 vLocalRight;

	Matrix3 rotmx;
	rotmx.rotate(scanPose.rot);
    rotmx.translate(scanPose.x, scanPose.y);
	//rotmx.fromPose2D(scanPose);
    //rotmx.setRotation(scanPose.rot);
    //rotmx.setTranslation(scanPose.x, scanPose.y);
	
	QPoint posePoint(scanPose.x, scanPose.y);
	
	step = M_PI/(scan->length-1)/3;
	
	int increment = 1;
	if( scan->length>181 )
		increment = 2;
	
    for (uint i = 0; i < scan->length; i+=increment) {
        a = M_PI/(scan->length-1)*i - M_PI_2;
        
       
        vLocalLeft = rotmx.multiply( Vector2( scan->range[i] * cos(a-step), scan->range[i] * sin(a-step) ) );

        vLocalMid = rotmx.multiply( Vector2( scan->range[i] * cos(a), scan->range[i] * sin(a) ) );

        vLocalRight = rotmx.multiply( Vector2( scan->range[i] * cos(a-step), scan->range[i] * sin(a-step) ) );
             
        painter.setPen(freeAreaPen);
        painter.drawLine(posePoint, QPoint(vLocalLeft.x, vLocalLeft.y));
        painter.drawLine(posePoint, QPoint(vLocalMid.x, vLocalMid.y));
        painter.drawLine(posePoint, QPoint(vLocalRight.x, vLocalRight.y));
          
        painter.setPen(occupiedAreaPen);
        painter.drawPoint(vLocalMid.x, vLocalMid.y);
        painter.drawPoint(vLocalMid.x, vLocalMid.y);
        
    }
}
