#include <QtDebug>

#include "ScanUtils.h"

Bounds operator+(const Bounds& b1, const Bounds& b2) {
    float bottom1 = b1.top  + b1.height;
    float bottom2 = b2.top  + b2.height;
    float right1  = b1.left + b1.width;
    float right2  = b2.left + b2.width;

    Bounds ans;
    ans.top  = b1.top  < b2.top  ? b1.top  : b2.top;
    ans.left = b1.left < b2.left ? b1.left : b2.left;
    ans.height = bottom1 > bottom2 ? bottom1 - ans.top  : bottom2 - ans.top;
    ans.width  = right1  > right2  ? right1  - ans.left : right2  - ans.left;
    return ans;
}


Matrix2 Matrix2::operator*(Matrix2& m) {
	Matrix2 ans;
	
	double sum=0;
	for (int i = 0; i < 2; i++) {
		for(int j = 0; j < 2; j++) {
			sum = 0;
			for(int k = 0; k < 2; k++)
				sum += (*this)(i,k) * m(k,j);
			ans(i,j) = sum;
		}
	}
	
	return ans;
}
Vector2 Matrix2::operator*(const Vector2& v) {
	Vector2 ans;
	ans.x = v.x * nums[0] + v.y * nums[1];
	ans.y = v.x * nums[2] + v.y * nums[3];
	return ans;
}
Matrix2 Matrix2::operator-(const Matrix2 & right)
{
	Matrix2 ans;
	float diff=0;
	for (int i = 0; i < 2; i++) {
		for(int j = 0; j < 2; j++) {
            diff = (*this)(i,j) - right.nums[index(i,j,2)];
			ans(i,j) = diff;
		}
	}
	return ans;
}
Matrix2 Matrix2::operator+(const Matrix2 & right)
{
	Matrix2 ans;
	float sum=0;
	for (int i = 0; i < 2; i++) {
		for(int j = 0; j < 2; j++) {
            sum = (*this)(i,j) + right.nums[index(i,j,2)];
			ans(i,j) = sum;
		}
	}
	return ans;
}

Matrix2 Matrix2::inverse() {
    Matrix2 ans;
    float det = nums[0]*nums[3] - nums[1]*nums[2];
    ans(0,0) = 1/det * nums[3];
    ans(0,1) = 1/det * -nums[1];
    ans(1,0) = 1/det * -nums[2];
    ans(1,1) = 1/det * nums[0];
    return ans;
}

Matrix2 Matrix2::transpose() {
    Matrix2 ans;
    ans(0,0) = nums[0];
    ans(0,1) = nums[2];
    ans(1,0) = nums[1];
    ans(1,1) = nums[3];
    return ans;
}

void Matrix2::loadIdentity() {
	for(int i = 0; i < 2; i++)
		for(int j = 0; j < 2; j++)
			if(i != j)
				nums[index(i,j,2)] = 0;
			else
				nums[index(i,j,2)] = 1;
}


Matrix2& Matrix2::operator=(const Matrix2& m) {
	for(int i = 0; i < 4; i++)
		nums[i] = m.nums[i];
	return *this;
}
Vector2 operator*(const Vector2 & v, const Matrix2 & m) {
	Vector2 ans;
	ans.x = v.x * m.nums[0] + v.y * m.nums[2];
	ans.y = v.x * m.nums[1] + v.y * m.nums[3];
	return ans;
}
Matrix2 operator*(const float & f, const Matrix2 & m) {
	Matrix2 ans;
	ans.nums[0] = m.nums[0] * f;
	ans.nums[1] = m.nums[1] * f;
	ans.nums[2] = m.nums[2] * f;
	ans.nums[3] = m.nums[3] * f;
	return ans;
}




Matrix3 Matrix3::operator+(const Matrix3 & right)
{
	Matrix3 ans;
	float sum=0;
	for (int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
            sum = (*this)(i,j) + right.nums[index(i,j,3)];
			ans(i,j) = sum;
		}
	}
	return ans;
}
Matrix3 Matrix3::operator*(const Matrix3& m) {
	Matrix3 ans;
	
	double sum=0;
	for (int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			sum = 0;
			for(int k = 0; k < 3; k++)
				sum += (*this)(i,k) * m.nums[index(k,j,3)];
			ans(i,j) = sum;
		}
	}
	
	return ans;
}

void Matrix3::loadIdentity() {
	for(int i = 0; i < 3; i++)
		for(int j = 0; j < 3; j++)
			if(i != j)
				nums[index(i,j,3)] = 0;
			else
				nums[index(i,j,3)] = 1;
}


Matrix3 Matrix3::transpose() {
    Matrix3 ans;
    ans(0,0) = (*this)(0, 0);
    ans(1,0) = (*this)(0, 1);
    ans(2,0) = (*this)(0, 2);
    ans(0,1) = (*this)(1, 0);
    ans(1,1) = (*this)(1, 1);
    ans(2,1) = (*this)(1, 2);
    ans(0,2) = (*this)(2, 0);
    ans(1,2) = (*this)(2, 1);
    ans(2,2) = (*this)(2, 2);
    return ans;
}



void Matrix3::rotate(float angle) {
	Matrix3 rot;
	rot.setRotation(angle);
	(*this) = rot.multiply((*this));
}

void Matrix3::translate(float x, float y) {
	Matrix3 t;
	t.setTranslation(x, y);
	(*this) = t.multiply((*this));
}

void Matrix3::invRotate(float angle) {
	Matrix3 rot;
	rot.setInvRotation(angle);
	(*this) = this->multiply(rot);
}

void Matrix3::invTranslate(float x, float y) {
	Matrix3 t;
	t.setTranslation(-x,-y);
	(*this) = this->multiply(t);
}

void Matrix3::setRotation(float angle) {
	float c = cos(angle);
	float s = sin(angle);
	
	(*this)(0,0) = c;
	(*this)(0,1) = -s;
	(*this)(1,0) = s;
	(*this)(1,1) = c;
}

void Matrix3::setInvRotation(float angle) {
	float c = cos(angle);
	float s = sin(angle);
	
	// actually transpose of rotation matrix
	(*this)(0,0) = c;
	(*this)(0,1) = s;
	(*this)(1,0) = -s;
	(*this)(1,1) = c;
}

void Matrix3::setTranslation(float x, float y) {
	(*this)(0,2) = x;
	(*this)(1,2) = y;
}

Vector2 Matrix3::multiply(const Vector2 &v) {
	Vector2 ans;
	ans.x = v.x*(*this)(0,0) + v.y*(*this)(0,1) + (*this)(0,2);
	ans.y = v.x*(*this)(1,0) + v.y*(*this)(1,1) + (*this)(1,2);
	return ans;
}

Matrix3 Matrix3::multiply(Matrix3 &m) {
	Matrix3 ans;
	
	double sum=0;
	for (int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			sum = 0;
			for(int k = 0; k < 3; k++)
				sum += (*this)(i,k) * m(k,j);
			ans(i,j) = sum;
		}
	}
	
	return ans;
}

Matrix3& Matrix3::operator=(const Matrix3& m) {
	for(int i = 0; i < 9; i++)
		nums[i] = m.nums[i];
	return *this;
}


Vector2 operator*(const float& left, const Vector2 & right)
{
	Vector2 temp(left*right.x, left*right.y);
	return temp;
}
float operator*(const Vector2& left, const Vector2 & right)
{
    float ans = (left.x * right.x + left.y * right.y);
    return ans;
}

Vector2 Vector2::operator-(const Vector2 & right)
{
	Vector2 temp(x-right.x, y-right.y);
	return temp;
}

Vector2 Vector2::operator+(const Vector2 & right)
{
	Vector2 temp(x+right.x,y+right.y);
	return temp;
}

Vector2& Vector2::operator=(const Vector2 & right)
{
	x = right.x;
	y = right.y;
	return *this;
}


ScanObservation::ScanObservation(QUuid id) {
	this->uniqueID = id;
	
	this->range = NULL;
}

ScanObservation::ScanObservation(float unit, float x, float y, float rot, LaserData laser) {

    this->uniqueID = QUuid::createUuid();

    // convert meters to millimeters
    Pose2D pose;
    pose.x   = x * unit;
    pose.y   = y * unit;
    pose.rot = rot;

    this->rawpose     = pose;
    this->pose        = pose;
    this->unit        = unit;
    this->length      = laser.length;
	this->maxdistance = laser.maxRange * unit;
	this->mindistance = laser.minRange * unit;

    this->range   = new float[laser.length];

    for (int i = 0; i < laser.length; i++) {
        this->range[i] = laser.data[i] * unit;
    }
}

ScanObservation::ScanObservation(float unit, Pose2D rawpose, Pose2D pose, float* range, double minDistance, double maxDistance, uint length) {
    
    this->uniqueID = QUuid::createUuid();

    this->rawpose     = rawpose;
    this->pose        = pose;
    this->unit        = unit;
    this->length      = length;
    this->range       = new float[length];
	this->mindistance = minDistance;
	this->maxdistance = maxDistance;

    for (uint i = 0; i < length; i++) {
        this->range[i] = range[i];
    }
}

ScanObservation::~ScanObservation() {
    delete[] range;
    range = 0;
}

QUuid ScanObservation::getUniqueID() {
    return this->uniqueID;
}
QMapIterator< QString, RFIDData > ScanObservation::rfidIterator() {
    QMapIterator< QString, RFIDData > iterator(this->rfids);
    return iterator;
}
QMapIterator< QString, VictimRFIDData > ScanObservation::victimIterator() {
    QMapIterator< QString, VictimRFIDData > iterator(this->victims);
    return iterator;
}
void ScanObservation::addRFID(RFIDData rfid) {
    if (!this->rfids.contains(rfid.ID)) {
        this->rfids.insert(rfid.ID, rfid);
    }
}
void ScanObservation::addVictim(VictimRFIDData victim) {
    if (!this->victims.contains(victim.ID)) {
        this->victims.insert(victim.ID, victim);
    }
}



MatchResult::MatchResult(Pose2D rawdpose, Pose2D smdpose, Matrix3 mcov, uint numIterations, uint numCorrespondencies, uint numPatchesInSubmap) {
	this->rawdpose            = rawdpose;
	this->smdpose             = smdpose;
	this->mcov                = mcov;
    this->numIterations       = numIterations;
    this->numCorrespondencies = numCorrespondencies;
    this->numPatchesInSubmap  = numPatchesInSubmap;
}

